@@ -194,36 +194,45 @@ int xs_jit_is_cached(pTHX_ const char *code, const char *name,
     return stat(path, &st) == 0;
 }
 
-/* Generate XS wrapper for a single function */
+/* Generate XS wrapper/alias for a single function */
 static int generate_wrapper(StrBuf *buf, const char *target,
-                            const char *source, int has_varargs) {
+                            const char *source, int has_varargs, int is_xs_native) {
     char safe_target[256];
     safe_name(target, safe_target, sizeof(safe_target));
 
-    strbuf_appendf(buf, "\n/* XS wrapper for %s -> %s */\n", source, target);
-    strbuf_appendf(buf, "XS_EUPXS(XS_%s) {\n", safe_target);
-    strbuf_append(buf, "    dVAR; dXSARGS;\n");
+    if (is_xs_native) {
+        /* For XS-native functions, just create an alias - no wrapper needed.
+         * The user function already has proper XS signature and handles
+         * dXSARGS, ST(), XSRETURN, etc. itself.
+         */
+        strbuf_appendf(buf, "\n/* Alias for XS-native %s -> %s */\n", source, target);
+        strbuf_appendf(buf, "#define XS_%s %s\n", safe_target, source);
+    } else {
+        strbuf_appendf(buf, "\n/* XS wrapper for %s -> %s */\n", source, target);
+        strbuf_appendf(buf, "XS_EUPXS(XS_%s) {\n", safe_target);
+        strbuf_append(buf, "    dVAR; dXSARGS;\n");
 
-    if (has_varargs) {
-        strbuf_append(buf, "    if (items < 1)\n");
-        strbuf_appendf(buf, "        croak_xs_usage(cv, \"self, ...\");\n");
+        if (has_varargs) {
+            strbuf_append(buf, "    if (items < 1)\n");
+            strbuf_appendf(buf, "        croak_xs_usage(cv, \"self, ...\");\n");
+        }
+
+        strbuf_append(buf, "    {\n");
+        strbuf_append(buf, "        SV *RETVAL;\n");
+        strbuf_append(buf, "        I32* temp = PL_markstack_ptr++;\n");
+        strbuf_appendf(buf, "        RETVAL = %s(ST(0));\n", source);
+        strbuf_append(buf, "        PL_markstack_ptr = temp;\n");
+        strbuf_append(buf, "        if (RETVAL) {\n");
+        strbuf_append(buf, "            RETVAL = sv_2mortal(RETVAL);\n");
+        strbuf_append(buf, "        } else {\n");
+        strbuf_append(buf, "            RETVAL = &PL_sv_undef;\n");
+        strbuf_append(buf, "        }\n");
+        strbuf_append(buf, "        ST(0) = RETVAL;\n");
+        strbuf_append(buf, "    }\n");
+        strbuf_append(buf, "    XSRETURN(1);\n");
+        strbuf_append(buf, "}\n");
     }
 
-    strbuf_append(buf, "    {\n");
-    strbuf_append(buf, "        SV *RETVAL;\n");
-    strbuf_append(buf, "        I32* temp = PL_markstack_ptr++;\n");
-    strbuf_appendf(buf, "        RETVAL = %s(ST(0));\n", source);
-    strbuf_append(buf, "        PL_markstack_ptr = temp;\n");
-    strbuf_append(buf, "        if (RETVAL) {\n");
-    strbuf_append(buf, "            RETVAL = sv_2mortal(RETVAL);\n");
-    strbuf_append(buf, "        } else {\n");
-    strbuf_append(buf, "            RETVAL = &PL_sv_undef;\n");
-    strbuf_append(buf, "        }\n");
-    strbuf_append(buf, "        ST(0) = RETVAL;\n");
-    strbuf_append(buf, "    }\n");
-    strbuf_append(buf, "    XSRETURN(1);\n");
-    strbuf_append(buf, "}\n");
-
     return 1;
 }
 
@@ -388,7 +397,7 @@ char* xs_jit_generate_code(pTHX_ const char *user_code,
     for (int i = 0; i < num_funcs; i++) {
         if (!functions[i].target) break;
         generate_wrapper(&buf, functions[i].target, functions[i].source,
-                        functions[i].has_varargs);
+                        functions[i].has_varargs, functions[i].is_xs_native);
     }
 
     /* Generate boot function */
